# -*- coding: utf-8 -*-
{
  "name": "Generationkwh production manager",
  "description": """Support for SomEnergia's plantmeter in GisceERP""",
  "version": "0.0.3",
  "author": "Som-Energia",
  "category": "Master",
  "depends": ['base'],
  "init_xml": [],
  "demo_xml": [],
  "update_xml": [
    "security/som_plantmeter.xml",
    "security/ir.model.access.csv",
    ],
  "active": False,
  "installable": True
}



import datetime
from ..providers import BaseProviderConnectionError, \
        BaseProviderDownloadError, BaseProviderSyntaxError
from .monsol import MonsolProvider, parseLocalTime
from ..isodates import isodate

import unittest
from mock import patch

class MonsolProvide_Test(unittest.TestCase):
    def setUp(self):
        ""

    def tearDown(self):
        ""

    def load_file(self, path):
        with open(path, 'r') as f:
            output = f.read().splitlines()
        return output

    def getContent(self, filename):
        import os
        path = os.path.dirname(os.path.realpath(__file__)) + '/../data/monsol/' + filename
        return self.load_file(path)

    def profileFromDate(self, start, hours, profile_kwh, daylight):
        return [
            {
            'ae': profile_kwh[idx],
            'datetime': parseLocalTime(
                    start + str(hour).zfill(2) + '0000' , daylight[idx]=='S')
            } for idx, hour in enumerate(hours)]

    def test_failHostname(self):
        m = MonsolProvider('monsol://user:password@hostname/path')
        self.assertRaises(BaseProviderConnectionError, m.download, isodate('2015-08-01'))

    def test_failConnection(self):
        from socket import error
        m = MonsolProvider('monsol://user:password@www.somenergia.coop/path')
        self.assertRaises(BaseProviderConnectionError, m.download, isodate('2015-08-01'))

    @patch('ftplib.FTP', autospec=True)
    def test_download(self, ftp_constructor):
        mock_ftp = ftp_constructor.return_value
        m = MonsolProvider('monsol://user:password@hostname/path')
        m.download(isodate('2015-08-01'))
    
        # TODO: Implement callback mockup
        mock_ftp.login.assert_called_once_with('user', 'password')
        mock_ftp.cwd.assert_called_once_with('path')
        self.assertEqual(mock_ftp.retrbinary.call_args[0], ('RETR 1_20150801.csv',))

    def test_extractEmptyFile(self):
        content = self.getContent('20_20160327_empty.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        measurements = m.extract(content, datetime.date(2015,9,4))
        self.assertEqual(measurements, [])

    def test_extractOnlyHeaderFile(self):
        content = self.getContent('20_20160327_header.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        measurements = m.extract(content, datetime.date(2015,9,4))
        self.assertEqual(measurements, [])

    def test_extractFileWinter(self):
        content = self.getContent('20_20160326.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        actual = m.extract(content, datetime.date(2016,3,26))
        expected = self.profileFromDate(
                '20160326',
                range(0,24),
                [0,0,0,0,0,0,0,0,0,7,14,21,29,24,26,26,28,23,16,6,0,0,0,0],
                ['W']*24)
        self.assertItemsEqual(actual, expected)

    def test_extractFileWinterToSummer(self):
        content = self.getContent('20_20160327.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        actual = m.extract(content, datetime.date(2016,3,27))
        expected = self.profileFromDate(
                '20160327',
                [0,1]+range(3,24),
                [0,0,0,0,0,0,0,0,9,15,20,27,29,31,31,30,26,11,8,0,0,0,0],
                ['W']*2+['S']*21)
        self.assertItemsEqual(actual, expected)

    def test_extractFileSummer(self):
        content = self.getContent('20_20160328.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        actual = m.extract(content, datetime.date(2016,3,28))
        expected = self.profileFromDate(
                '20160328',
                range(0,24),
                [0,0,0,0,0,0,0,0,0,10,11,22,26,31,30,30,29,25,12,9,0,0,0,0],
                ['S']*24)
        self.assertItemsEqual(actual, expected)

    def test_wronglinesize(self):
        content = self.getContent('20_20160328_wrongLineSize.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        with self.assertRaises(BaseProviderSyntaxError):
            m.extract(content, datetime.date(2016,3,28))

    def test_wrongMissingDatetime(self):
        content = self.getContent('20_20160328_missingDate.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        with self.assertRaises(BaseProviderSyntaxError):
            m.extract(content, datetime.date(2016,3,28))

    def test_missingExportedEnergy(self):
        content = self.getContent('20_20160328_missingExportedEnergy.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        with self.assertRaises(BaseProviderSyntaxError):
            m.extract(content, datetime.date(2016,3,28))

    def test_missingImportedEnergy(self):
        content = self.getContent('20_20160328_missingImportedEnergy.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        with self.assertRaises(BaseProviderSyntaxError):
            m.extract(content, datetime.date(2016,3,28))

    def test_missingFormatDaylight(self):
        content = self.getContent('20_20160328_missingImportedEnergy.csv')
        m = MonsolProvider('monsol://user:password@hostname/path')
        with self.assertRaises(BaseProviderSyntaxError):
            m.extract(content, datetime.date(2016,3,28))


# vim: et ts=4 sw=4



# -*- coding: utf-8 -*-

from plantmeter.providers import BaseProvider, register, urlparse
from plantmeter.utils import daterange
from plantmeter.isodates import localisodatetime, assertDateOrNone
import datetime
import csv
from ..mongotimecurve import parseLocalTime, toLocal

class CSVProvider(BaseProvider):
    """CSV provider
    """

    def __init__(self, uri=None):
        if uri is None:
            uri = "csv://path"
        super(CSVProvider, self).__init__(uri)
        self.uri = uri
        self.res = self.uri.split(':')[1]

    def get(self, start, end=None):
        # TODO: Don't let this code survive!!

        assertDateOrNone('start', start)
        assertDateOrNone('end', end)

        def extract(line):
            items = line.rstrip().split(';')
            return {
                'datetime': parseLocalTime(items[0]+':00', items[1]=='S'),
                'ae': items[2]
                }

        if not end:
            end = toLocal(datetime.datetime.now()).date()
        with open(self.res, 'rb') as csvfile:
            content = csvfile.readlines()
            return [
                [
                measure
                for measure in (
                    extract(line)
                    for line in content[1:]
                    )
                if measure['datetime'].date() == date # TODO: la mare dels ous
                ]
                for date in daterange(start, end + datetime.timedelta(days=1))
            ]

    def disconnect(self):
        pass

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

register("csv", CSVProvider)

# vim: et ts=4 sw=4



# -*- coding: utf-8 -*-
"""
    plantmeter.logging
    ~~~~~~~~~~~~~~~

    Implements the logging support for plantmeter 

    You can use logging everywhere using::

        from plantmeter import logger
        logger.info('Info message')
"""
from __future__ import absolute_import

import logging

#from raven import Client as SentryClient
#from raven.handlers.logging import SentryHandler
from plantmeter import VERSION

LOG_FORMAT = '[%(asctime)s] %(levelname)s in %(module)s: %(message)s'


def setup_logging(level=None, logfile=None):
    """
    Setups plantmeter logging system.

    It will setup sentry logging if SENTRY_DSN environment is defined

    :param level: logging.LEVEL to set to logger (defaults INFO)
    :param logfile: File to write the log
    :return: logger
    """
    stream = logging.StreamHandler()
    stream.setFormatter(logging.Formatter(LOG_FORMAT))

    logger = logging.getLogger('plantmeter')
    del logger.handlers[:]
    logger.addHandler(stream)

    if logfile:
        hdlr = logging.FileHandler(logfile)
        formatter = logging.Formatter(LOG_FORMAT)
        hdlr.setFormatter(formatter)
        logger.addHandler(hdlr)

    #sentry = SentryClient()
    #sentry.tags_context({'version': VERSION})
    #sentry_handler = SentryHandler(sentry, level=logging.ERROR)
    #logger.addHandler(sentry_handler)

    if isinstance(level, basestring):
        level = getattr(logging, level.upper(), None)

    if level is None:
        level = logging.INFO

    logger.setLevel(level)

    return logger



#!/usr/bin/env python

from .mongotimecurve import (
    MongoTimeCurve,
    dateToCurveIndex,
    curveIndexToDate,
    parseLocalTime,
    tz,
    assertLocalDateTime,
    asUtc,
    toLocal,
    )
import pymongo
import datetime

import unittest

from .isodates import localisodate

def localTime(string):
    isSummer = string.endswith("S")
    if isSummer: string=string[:-1]
    return parseLocalTime(string, isSummer)

class LocalTime_Test(unittest.TestCase):

    def test_localTime_fullySummer(self):
        self.assertEqual(
            str(localTime("2016-08-15 02:00:00")),
            "2016-08-15 02:00:00+02:00")

    def test_localTime_fullyWinter(self):
        self.assertEqual(
            str(localTime("2016-01-01 02:10:00")),
            "2016-01-01 02:10:00+01:00")

    def test_localTime_badTz_ignored(self):
        self.assertEqual(
            str(localTime("2016-01-01 02:00:00S")),
            "2016-01-01 02:00:00+01:00")

    def test_localTime_badSummerTz_ignored(self):
        self.assertEqual(
            str(localTime("2016-08-15 02:00:00")),
            "2016-08-15 02:00:00+02:00")

    def test_localTime_beforeOctoberChange(self):
        self.assertEqual(
            str(localTime("2016-10-30 02:00:00S")),
            "2016-10-30 02:00:00+02:00")

    def test_localTime_afterOctoberChange(self):
        self.assertEqual(
            str(localTime("2016-10-30 02:00:00")),
            "2016-10-30 02:00:00+01:00")

    def test_localTime_SIgnored(self):
        self.assertEqual(
            str(localTime("2016-10-30 03:00:00S")),
            "2016-10-30 03:00:00+01:00")

    def test_localTime_unexistingHour(self):
        # TOREVIEW: should it fail
        self.assertEqual(
            str(localTime("2016-03-27 02:00:00")),
            "2016-03-27 02:00:00+01:00")

    def test_localTime_atWinterMidnight(self):
        self.assertEqual(
            str(localTime("2016-01-01 00:00:00")),
            "2016-01-01 00:00:00+01:00")

    def test_assertLocalDateTime_withDate(self):
        with self.assertRaises(AssertionError) as ctx:
            assertLocalDateTime('myname', datetime.date(2016,01,01))
        self.assertEqual(ctx.exception.args[0],
            "myname should be a datetime")

    def test_assertLocalDateTime_withNaive(self):
        with self.assertRaises(AssertionError) as ctx:
            assertLocalDateTime('myname', datetime.datetime(2016,01,01))
        self.assertEqual(ctx.exception.args[0],
            "myname should have timezone")

    def test_assertLocalDateTime_withUTC(self):
        with self.assertRaises(AssertionError) as ctx:
            assertLocalDateTime('myname', asUtc(datetime.datetime(2016,01,01)))
        self.assertEqual(ctx.exception.args[0],
            "myname has UTC timezone")

    def test_assertLocalDateTime_withLocal(self):
        assertLocalDateTime('myname', toLocal(datetime.datetime(2016,01,01)))
        # No assert


class CurveDatetimeMapper_Test(unittest.TestCase):

    def assertDateEqual(self, result, expected):
        self.assertEqual(result, expected)
        self.assertEqual(str(result), str(expected))

    def test_dateToCurveIndex_inSummer(self):
       self.assertEquals(
            dateToCurveIndex(
                localisodate("2016-08-15"),
                localTime("2016-08-15 00:00:00")
                ), 0)

    def test_dateToCurveIndex_inSummer_secondHour(self):
       self.assertEquals(
            dateToCurveIndex(
                localisodate("2016-08-15"),
                localTime("2016-08-15 01:00:00")
                ), 1)

    def test_dateToCurveIndex_inSummer_nextDay(self):
       self.assertEquals(
            dateToCurveIndex(
                localisodate("2016-08-15"),
                localTime("2016-08-16 00:00:00")
                ), 25)

    def test_dateToCurveIndex_inWinter(self):
       self.assertEquals(
            dateToCurveIndex(
                localisodate("2016-12-25"),
                localTime("2016-12-25 00:00:00")
                ), 1)

    def test_dateToCurveIndex_beforeSummerToWinterChange(self):
       self.assertEquals(
            dateToCurveIndex(
                localisodate("2016-10-30"),
                localTime("2016-10-30 02:00:00S")
                ), 2)

    def test_dateToCurveIndex_afterSummerToWinterChange(self):
       self.assertEquals(
            dateToCurveIndex(
                localisodate("2016-10-30"),
                localTime("2016-10-30 02:00:00")
                ), 3)

    def test_dateToCurveIndex_beforeWinterToSummerChange(self):
       self.assertEquals(
            dateToCurveIndex(
                localisodate("2016-03-27"),
                localTime("2016-03-27 01:00:00")
                ), 2)

    def test_dateToCurveIndex_afterWinterToSummerChange(self):
       self.assertEquals(
            dateToCurveIndex(
                localisodate("2016-03-27"),
                localTime("2016-03-27 03:00:00")
                ), 3)

    def test_curveIndexToDate_summer(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-08-15"), 0),
            localTime("2016-08-15 00:00:00"))

    def test_curveIndexToDate_summer_secondHour(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-08-15"), 1),
            localTime("2016-08-15 01:00:00"))

    def test_curveIndexToDate_summer_nextDay(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-08-15"), 25),
            localTime("2016-08-16 00:00:00"))

    def test_curveIndexToDate_summer_paddingReturnsNone(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-08-15"), 24),
            None)

    def test_curveIndexToDate_winter(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-12-25"), 1),
            localTime("2016-12-25 00:00:00"))

    def test_curveIndexToDate_winter_paddingReturnsNone(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-12-25"), 0),
            None)

    def test_curveIndexToDate_winter_nextDay(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-12-25"), 26),
            localTime("2016-12-26 00:00:00"))

    
    def test_curveIndexToDate_beforeChanginToWinter(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-10-30"), 2),
            localTime("2016-10-30 02:00:00S"))

    def test_curveIndexToDate_afterChanginToWinter(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-10-30"), 3),
            localTime("2016-10-30 02:00:00"))

    def test_curveIndexToDate_afterChanginToWinter(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-10-30"), 3),
            localTime("2016-10-30 02:00:00"))

    def test_curveIndexToDate_summerButStartIsWinter(self):
       self.assertDateEqual(
            curveIndexToDate(localisodate("2016-10-29"), 51),
            localTime("2016-10-31 00:00:00"))

    def test_curveIndexToDate_winterButStartIsSummer(self):
        self.assertDateEqual(
            curveIndexToDate(localisodate("2016-3-26"), 50),
            localTime("2016-3-28 00:00:00"))

    def test_curveIndexToDate_noPaddingBeforeSummerToWinter(self):
        self.assertDateEqual(
            curveIndexToDate(localisodate("2016-10-30"), 0),
            localTime("2016-10-30 00:00:00"))

    def test_curveIndexToDate_noPaddingAfterSummerToWinter(self):
        self.assertDateEqual(
            curveIndexToDate(localisodate("2016-10-30"), 24),
            localTime("2016-10-30 23:00:00"))

    def test_curveIndexToDate_paddingBeforeWinterToSummer(self):
        self.assertDateEqual(
            curveIndexToDate(localisodate("2016-03-27"), 0),
            None)

    def test_curveIndexToDate_paddingAfterWinterToSummer(self):
        self.assertDateEqual(
            curveIndexToDate(localisodate("2016-03-27"), 24),
            None)

    

class MongoTimeCurve_Test(unittest.TestCase):

    def setUp(self):
        self.databasename = 'generationkwh_test'
        self.collection = 'generation'

        c = pymongo.MongoClient()
        c.drop_database(self.databasename)
        self.db = c[self.databasename]

    def tearDown(self):
        c = pymongo.MongoClient()
        c.drop_database('generationkwh_test')
    
    def setupPoints(self, points):
        mtc = MongoTimeCurve(self.db, self.collection)
        for datetime, plant, value in points:
            mtc.fillPoint(
                datetime=localTime(datetime),
                name=plant,
                ae=value,
            )
        return mtc


    def test_get_whenEmpty(self):
        mtc = self.setupPoints([
            ])
        curve = mtc.get(
            start=localisodate('2015-01-01'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            [0]*25)

    def test_get_withSingleData(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'miplanta', 10),
            ])

        curve = mtc.get(
            start=localisodate('2015-01-01'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            [0]*24+[10])

    def test_get_twoDays(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'miplanta', 10),
            ])

        curve = mtc.get(
            start=localisodate('2014-12-31'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            +25*[0]
            +24*[0]+[10]
            )

    def test_get_differentTime(self):
        mtc = self.setupPoints([
            ('2015-01-01 22:00:00', 'miplanta', 10),
            ])

        curve = mtc.get(
            start=localisodate('2015-01-01'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            str(list(curve)),
            str(+23*[0]+[10,0])
            )

    def test_get_withOutsideData(self):
        mtc = self.setupPoints([
            ('2015-01-02 23:00:00', 'miplanta', 10),
            ])

        curve = mtc.get(
            start=localisodate('2015-01-01'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            +25*[0])

    def test_get_withTwoPoints(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'miplanta', 10),
            ('2015-01-01 01:00:00', 'miplanta', 10),
            ])

        curve = mtc.get(
            start=localisodate('2015-01-01'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            [0,0,10]+21*[0]+[10])

    def test_get_differentNameIgnored(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'otraplanta', 10),
            ])

        curve = mtc.get(
            start=localisodate('2015-01-01'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            +25*[0])


    def test_get_prioritizesNewest(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'miplanta', 10),
            ('2015-01-01 23:00:00', 'miplanta', 30),
            ])

        curve = mtc.get(
            start=localisodate('2015-01-01'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            +24*[0]+[30])

    def test_get_summerPointsPadsLeft(self):
        mtc = self.setupPoints([
            ('2015-08-01 0:00:00', 'miplanta', 20),
            ('2015-08-01 23:00:00', 'miplanta', 10),
            ])

        curve = mtc.get(
            start=localisodate('2015-08-01'),
            stop=localisodate('2015-08-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            [20]+22*[0]+[10,0])

    def test_get_dayligthIntoSummer(self):
        mtc = self.setupPoints([
            ('2015-03-29 00:00:00', 'miplanta', 1),
            ('2015-03-29 01:00:00', 'miplanta', 2),
#            ('2015-03-29 02:00:00', 'miplanta', 2), # should fail (other test?)
            ('2015-03-29 03:00:00', 'miplanta', 3),
            ('2015-03-29 23:00:00', 'miplanta', 4),
            ])

        curve = mtc.get(
            start=localisodate('2015-03-29'),
            stop=localisodate('2015-03-29'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            [0,1,2,3]+19*[0]+[4,0])

    def test_get_twoDays_includingToSummerDaylight(self):
        mtc = self.setupPoints([
            ('2015-03-30 23:00:00', 'miplanta', 10),
            ])

        curve = mtc.get(
            start=localisodate('2015-03-29'),
            stop=localisodate('2015-03-30'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            +25*[0]
            +23*[0]+[10,0]
            )

    def test_get_dayligthIntoWinter(self):
        mtc = self.setupPoints([
            ('2015-10-25 00:00:00', 'miplanta', 1),
            ('2015-10-25 02:00:00S', 'miplanta', 2), # how to mark it??
            ('2015-10-25 02:00:00', 'miplanta', 3), # how to mark it??
            ('2015-10-25 23:00:00', 'miplanta', 4),
            ])

        curve = mtc.get(
            start=localisodate('2015-10-25'),
            stop=localisodate('2015-10-25'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            [1,0,2,3]+20*[0]+[4])

    def test_fillPoint_complaintsMissingDatetime(self):
        mtc = MongoTimeCurve(self.db, self.collection)
        with self.assertRaises(Exception) as ass:
            mtc.fillPoint(
                name='miplanta',
                ae=10,
                )
        self.assertEqual(ass.exception.args[0],
            "Missing 'datetime'")

    def test_fillPoint_complaintsMissingName(self):
        mtc = MongoTimeCurve(self.db, self.collection)
        with self.assertRaises(Exception) as ass:
            mtc.fillPoint(
                datetime=localTime('2015-08-01 23:00:00'),
                ae=10,
                )
        self.assertEqual(ass.exception.args[0],
            "Missing 'name'")

    def test_lastDate_whenNoPoint_returnsNone(self):
        mtc = self.setupPoints([
            ])

        lastdate = mtc.lastDate('miplanta')
        self.assertEqual(lastdate,None)

    def test_lastDate_withOnePoint_takesIt(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'miplanta', 30),
            ])

        lastdate = mtc.lastDate('miplanta')
        self.assertEqual(lastdate,localisodate('2015-01-01'))

    def test_lastDate_withOnePoint_atMidnight(self):
        mtc = self.setupPoints([
            ('2015-01-01 00:00:00', 'miplanta', 30),
            ])

        lastdate = mtc.lastDate('miplanta')
        self.assertEqual(lastdate,localisodate('2015-01-01'))

    def test_lastDate_withForeignPoint_ignored(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'otraplanta', 30)
            ])

        lastdate = mtc.lastDate('miplanta')
        self.assertEqual(lastdate,None)
      
    def test_lastDate_withSeveralPoints_takesLast(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'miplanta', 30),
            ('2015-01-02 23:00:00', 'miplanta', 30),
            ])

        lastdate = mtc.lastDate('miplanta')
        self.assertEqual(lastdate,localisodate('2015-01-02'))

    def test_firstDate_whenNoPoint_returnsNone(self):
        mtc = self.setupPoints([
            ])

        lastdate = mtc.firstDate('miplanta')
        self.assertEqual(lastdate,None)

    def test_firstDate_withOnePoint_takesIt(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'miplanta', 30),
            ])

        lastdate = mtc.firstDate('miplanta')
        self.assertEqual(lastdate,localisodate('2015-01-01'))

    def test_firstDate_withForeignPoint_ignored(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'otraplanta', 30)
            ])

        lastdate = mtc.firstDate('miplanta')
        self.assertEqual(lastdate,None)

    def test_firstDate_withSeveralPoints_takesFirst(self):
        mtc = self.setupPoints([
            ('2015-01-01 23:00:00', 'miplanta', 30),
            ('2015-01-02 23:00:00', 'miplanta', 30),
            ])

        lastdate = mtc.firstDate('miplanta')
        self.assertEqual(lastdate,localisodate('2015-01-01'))

    def setupDatePoints(self, date, name, values):
        return self.setupPoints([
            (date+" {:02}:00:00".format(i), name, value)
            for i, value in enumerate(values)
            ])

    def test_setupDatePoints(self):
        mtc = self.setupDatePoints('2015-01-01', 'miplanta', range(1,25))

        curve = mtc.get(
            start=localisodate('2015-01-01'),
            stop=localisodate('2015-01-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            list([0]+range(1,25)))

    def test_lastFullDate_withNoPoints(self):
        mtc = self.setupDatePoints('2015-01-01', 'miplanta', [1]*24)

        lastdate = mtc.lastFullDate('miplanta')
        self.assertEqual(lastdate,localisodate('2015-01-01'))

    def test_firstFullDate_withNoPoints(self):
        mtc = self.setupDatePoints('2015-01-01', 'miplanta', [1]*24)

        firstdate = mtc.lastFullDate('miplanta')
        self.assertEqual(firstdate,localisodate('2015-01-01'))

    def test_filled_whenEmpty(self):
        mtc = self.setupPoints([])

        _, curve = mtc.get(
            start=localisodate('2015-08-15'),
            stop=localisodate('2015-08-15'),
            filter='miplanta',
            field='ae',
            filling=True,
            )
        self.assertEqual(
            list(curve),
            +25*[False]
            )

    def test_filled_withAPoint_inSummer(self):
        mtc = self.setupPoints([
            ('2015-08-15 00:00:00', 'miplanta', 30),
            ])

        _,curve = mtc.get(
            start=localisodate('2015-08-15'),
            stop=localisodate('2015-08-15'),
            filter='miplanta',
            field='ae',
            filling=True,
            )
        self.assertEqual(
            list(curve),
            [True]+24*[False]
            )

    def test_update_withNaiveDate(self):
        mtc = self.setupPoints([])

        with self.assertRaises(AssertionError) as ctx:
            mtc.update(
                start=datetime.datetime(2015,8,15),
                filter='miplanta',
                field='ae',
                data=[],
                )
        self.assertEqual(ctx.exception.args[0],
            "MongoTimeCurve.update called with naive (no timezone) start date")

    def test_fillPoint_withNaiveDatetime(self):
        mtc = self.setupPoints([])

        with self.assertRaises(AssertionError) as ctx:
            mtc.fillPoint(
                datetime=datetime.datetime(2015,8,15),
                name='miplanta',
                ae=10,
                )
        self.assertEqual(ctx.exception.args[0],
            "MongoTimeCurve.fillPoint with naive (no timezone) datetime")

    def test_get_withNaiveStartDate(self):
        mtc = self.setupPoints([])

        with self.assertRaises(AssertionError) as ctx:
            mtc.get(
                start=datetime.datetime(2015,8,15),
                stop=localisodate("2015-08-15"),
                filter='miplanta',
                field='ae',
                )
        self.assertEqual(ctx.exception.args[0],
            "MongoTimeCurve.get called with naive (no timezone) start date")

    def test_get_withNaiveStopDate(self):
        mtc = self.setupPoints([])

        with self.assertRaises(AssertionError) as ctx:
            mtc.get(
                start=localisodate("2015-08-15"),
                stop=datetime.datetime(2015,8,15),
                filter='miplanta',
                field='ae',
                )
        self.assertEqual(ctx.exception.args[0],
            "MongoTimeCurve.get called with naive (no timezone) stop date")

    def test_update_singleBin(self):
        mtc = self.setupPoints([])

        curve = mtc.update(
            start=localisodate('2015-08-15'),
            filter='miplanta',
            field='ae',
            data=[1]+24*[0]
            )
        curve, filling = mtc.get(
            start=localisodate('2015-08-15'),
            stop=localisodate('2015-08-15'),
            filter='miplanta',
            field='ae',
            filling=True,
            )
        self.assertEqual(
            list(curve),
            [1]+24*[0]
            )
        self.assertEqual(
            list(filling),
            [True]+24*[False]
            )

    def test_get_withFilledGap(self):
        mtc = self.setupPoints([])

        curve = mtc.update(
            start=localisodate('2015-08-01'),
            filter='miplanta',
            field='ae',
            data=+25*[1]+[1]+24*[1]
            )
        curve = mtc.get(
            start=localisodate('2015-08-01'),
            stop=localisodate('2015-08-01'),
            filter='miplanta',
            field='ae',
            )
        self.assertEqual(
            list(curve),
            +24*[1]+[0]
            )

    def test_update_fullDay(self):
        mtc = self.setupPoints([])

        curve = mtc.update(
            start=localisodate('2015-08-15'),
            filter='miplanta',
            field='ae',
            data=+25*[1]
            )
        curve, filling = mtc.get(
            start=localisodate('2015-08-15'),
            stop=localisodate('2015-08-15'),
            filter='miplanta',
            field='ae',
            filling=True,
            )
        self.assertEqual(
            list(curve),
            +24*[1]+[0]
            )
        self.assertEqual(
            list(filling),
            +24*[True]+[False]
            )




# vim: et ts=4 sw=4



#!/usr/bin/env python

from .mongotimecurve import MongoTimeCurve, tz
from plantmeter.resource import *
from plantmeter.isodates import localisodatetime,naiveisodatetime

import pymongo
from datetime import datetime, date

import unittest

def local_file(filename):
    return os.path.join(os.path.abspath(os.path.dirname(__file__)), filename)

class Resource_Test(unittest.TestCase):

    def setUp(self):
        self.databasename = 'generationkwh_test'
        self.collection = 'production'

        self.connection = pymongo.MongoClient()
        self.connection.drop_database(self.databasename)
        self.db = self.connection[self.databasename]
        self.curveProvider = MongoTimeCurve(self.db, self.collection)

    def tearDown(self):
        self.connection.drop_database('generationkwh_test')

    def test_getEmpty(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        curveProvider = self.curveProvider 
        m = ProductionMeter(
                1,
                'meterName',
                'meterDescription',
                True,
                uri=uri,
                curveProvider = self.curveProvider)

        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m)
        aggr = ProductionAggregator(1,'aggrName','eggrDescription',True)
        aggr.plants.append(p)

        self.assertEqual(
            list(aggr.get_kwh(
                date(2015,9,4),
                date(2015,9,5))),
                2*25*[0] 
                )
      
    def test_update_onePlantOneMeter(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        curveProvider = self.curveProvider 
        m = ProductionMeter(
                1,
                'meterName',
                'meterDescription',
                True,
                uri=uri,
                curveProvider = self.curveProvider)

        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m)
        aggr = ProductionAggregator(1,'aggrName','aggrDescription',True)
        aggr.plants.append(p)
        m.update_kwh(
            date(2015,9,4),
            date(2015,9,5))

        self.assertEqual(
            list(aggr.get_kwh(
                date(2015,9,4),
                date(2015,9,5))),
            [
                0,0,0,0,0,0,0,0,3,6,5,4,8,17,34,12,12,5,3,1,0,0,0,0,0,
                0,0,0,0,0,0,0,0,4,7,6,5,9,18,35,13,13,6,4,2,0,0,0,0,0,
            ])

    def test_update_onePlantTwoMeters(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        curveProvider = self.curveProvider 
        m1 = ProductionMeter(
                1,
                'meterName1',
                'meterDescription1',
                True,
                uri=uri,
                curveProvider = self.curveProvider)

        m2 = ProductionMeter(
                2,
                'meterName2',
                'meterDescription2',
                True,
                uri=uri,
                curveProvider = self.curveProvider)

        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m1)
        p.meters.append(m2)
        aggr = ProductionAggregator(1,'aggrName','aggrDescription',True)
        aggr.plants.append(p)
        m1.update_kwh(date(2015,9,4),date(2015,9,5))
        m2.update_kwh(date(2015,9,4),date(2015,9,5))

        self.assertEqual(
            list(aggr.get_kwh(
                date(2015,9,4),
                date(2015,9,5))),
            [
                0,0,0,0,0,0,0,0,6,12,10, 8,16,34,68,24,24,10,6,2,0,0,0,0,0,
                0,0,0,0,0,0,0,0,8,14,12,10,18,36,70,26,26,12,8,4,0,0,0,0,0,
            ])

    def test_update_twoPlantsOneMeter(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        curveProvider = self.curveProvider 
        m1 = ProductionMeter(
                1,
                'meterName1',
                'meterDescription1',
                True,
                uri=uri,
                curveProvider = self.curveProvider)
        p1 = ProductionPlant(1,'plantName1','plantDescription1',True)
        p1.meters.append(m1)
        p2 = ProductionPlant(2,'plantName2','plantDescription2',True)

        aggr = ProductionAggregator(1,'aggrName','aggrDescription',True)
        aggr.plants.append(p1)
        aggr.plants.append(p2)
        m1.update_kwh(date(2015,9,4),date(2015,9,5))

        self.assertEqual(
            list(aggr.get_kwh(
                date(2015,9,4),
                date(2015,9,5))),
            [
                0,0,0,0,0,0,0,0,3,6,5,4,8,17,34,12,12,5,3,1,0,0,0,0,0,
                0,0,0,0,0,0,0,0,4,7,6,5,9,18,35,13,13,6,4,2,0,0,0,0,0,
            ])

    def test_update_twoPlantsTwoMeters(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        curveProvider = self.curveProvider 
        m1 = ProductionMeter(
                1,
                'meterName1',
                'meterDescription1',
                True,
                uri=uri,
                curveProvider = self.curveProvider)
        p1 = ProductionPlant(1,'plantName1','plantDescription1',True)
        p1.meters.append(m1)

        m2 = ProductionMeter(
                1,
                'meterName2',
                'meterDescription2',
                True,
                uri=uri,
                curveProvider = self.curveProvider)
        p2 = ProductionPlant(2,'plantName2','plantDescription2',True)
        p2.meters.append(m2)

        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p1)
        aggr.plants.append(p2)
        m1.update_kwh(date(2015,9,4),date(2015,9,5))
        m2.update_kwh(date(2015,9,4),date(2015,9,5))

        self.assertEqual(
            list(aggr.get_kwh(
                date(2015,9,4),
                date(2015,9,5))),
            [
                0,0,0,0,0,0,0,0,6,12,10, 8,16,34,68,24,24,10,6,2,0,0,0,0,0,
                0,0,0,0,0,0,0,0,8,14,12,10,18,36,70,26,26,12,8,4,0,0,0,0,0,
            ])

    def test_update_startMissing(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        curveProvider = self.curveProvider
        m = ProductionMeter(
                1,
                'meterName',
                'meterDescription',
                True,
                uri=uri,
                lastcommit='2015-09-04',
                curveProvider = self.curveProvider)

        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m)
        aggr = ProductionAggregator(1,'aggrName','aggrDescription',True)
        aggr.plants.append(p)
        updated = aggr.update_kwh()
        # Check single aggregator, with single plant and meter
        self.assertEqual(updated[0][1][0][1], localisodatetime('2015-09-05 23:00:00'))

    def test_update_outofdate(self):
        uri = 'csv:/' + local_file('data/manlleu_20150804.csv')
        curveProvider = self.curveProvider
        m = ProductionMeter(
                1,
                'meterName',
                'meterDescription',
                True,
                uri=uri,
                lastcommit='2015-09-04',
                curveProvider = self.curveProvider)

        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m)
        aggr = ProductionAggregator(1,'aggrName','aggrDescription',True)
        aggr.plants.append(p)
        updated = aggr.update_kwh()
        # Check single aggregator, with single plant and meter
        self.assertEqual(updated[0][1][0][1], None)

    def test_lastDate_empty(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = uri,
            curveProvider = self.curveProvider)
        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m)
        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p)

        self.assertEqual(aggr.lastMeasurementDate(), None)
    
    def test_firstDate_empty(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = uri,
            curveProvider = self.curveProvider)
        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m)
        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p)

        self.assertEqual(aggr.firstMeasurementDate(), None)

    def test_lastDate_onePlantOneMeter(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = uri,
            curveProvider = self.curveProvider)
        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m)
        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p)
        m.update_kwh(date(2015,9,4), date(2015,9,5))

        self.assertEqual(aggr.lastMeasurementDate(), date(2015,9,5))

    def test_firstDate_onePlantOneMeter(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = uri,
            curveProvider = self.curveProvider)
        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m)
        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p)
        m.update_kwh(date(2015,9,4), date(2015,9,5))

        self.assertEqual(aggr.firstMeasurementDate(), date(2015,9,4))

    def test_lastDate_onePlantTwoMeters(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        m1 = ProductionMeter(
            1,
            'meterName1',
            'meterDescription1',
            True,
            uri = uri,
            curveProvider = self.curveProvider)
        uri = 'csv:/' + local_file('data/manlleu_20150804.csv')
        m2 = ProductionMeter(
            1,
            'meterName2',
            'meterDescription2',
            True,
            uri = uri,
            curveProvider = self.curveProvider)

        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m1)
        p.meters.append(m2)
        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p)
        m1.update_kwh(date(2015,9,4), date(2015,9,5))
        m2.update_kwh(date(2015,8,4), date(2015,8,5))

        self.assertEqual(aggr.lastMeasurementDate(), date(2015,9,5))

    def test_firstDate_onePlantTwoMeters(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        m1 = ProductionMeter(
            1,
            'meterName1',
            'meterDescription1',
            True,
            uri = uri,
            curveProvider = self.curveProvider)
        uri = 'csv:/' + local_file('data/manlleu_20150804.csv')
        m2 = ProductionMeter(
            2,
            'meterName2',
            'meterDescription2',
            True,
            uri = uri,
            curveProvider = self.curveProvider)

        p = ProductionPlant(1,'plantName','plantDescription',True)
        p.meters.append(m1)
        p.meters.append(m2)
        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p)
        m1.update_kwh(date(2015,9,4), date(2015,9,5))
        m2.update_kwh(date(2015,8,4), date(2015,8,5))

        self.assertEqual(aggr.firstMeasurementDate(), date(2015,8,4))

    def test_lastDate_twoPlantsTwoMeters(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        m1 = ProductionMeter(
            1,
            'meterName1',
            'meterDescription1',
            True,
            uri = uri,
            curveProvider = self.curveProvider)
        uri = 'csv:/' + local_file('data/manlleu_20150804.csv')
        m2 = ProductionMeter(
            2,
            'meterName2',
            'meterDescription2',
            True,
            uri = uri,
            curveProvider = self.curveProvider)

        p1 = ProductionPlant(1,'plantName1','plantDescription1',True)
        p2 = ProductionPlant(2,'plantName2','plantDescription2',True)
        p1.meters.append(m1)
        p2.meters.append(m2)
        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p1)
        aggr.plants.append(p2)
        m1.update_kwh(date(2015,9,4), date(2015,9,5))
        m2.update_kwh(date(2015,8,4), date(2015,8,5))

        self.assertEqual(aggr.lastMeasurementDate(), date(2015,9,5))

    def test_firstDate_twoPlantsTwoMeters(self):
        uri = 'csv:/' + local_file('data/manlleu_20150904.csv')
        m1 = ProductionMeter(
            1,
            'meterName1',
            'meterDescription1',
            True,
            uri = uri,
            curveProvider = self.curveProvider)
        uri = 'csv:/' + local_file('data/manlleu_20150804.csv')
        m2 = ProductionMeter(
            2,
            'meterName2',
            'meterDescription2',
            True,
            uri = uri,
            curveProvider = self.curveProvider)

        p1 = ProductionPlant(1,'plantName1','plantDescription1',True)
        p2 = ProductionPlant(2,'plantName2','plantDescription2',True)
        p1.meters.append(m1)
        p2.meters.append(m2)
        aggr = ProductionAggregator(1,'aggrName','aggreDescription',True)
        aggr.plants.append(p1)
        aggr.plants.append(p2)
        m1.update_kwh(date(2015,9,4), date(2015,9,5))
        m2.update_kwh(date(2015,8,4), date(2015,8,5))

        self.assertEqual(aggr.firstMeasurementDate(), date(2015,8,4))

class Meter_Test(unittest.TestCase):
    def setUp(self):
        self.databasename = 'generationkwh_test'
        self.collection = 'production'

        self.connection = pymongo.MongoClient()
        self.connection.drop_database(self.databasename)
        self.db = self.connection[self.databasename]
        self.curveProvider = MongoTimeCurve(self.db, self.collection)
        self.uri = 'csv:/' + local_file('data/manlleu_20150904.csv')

    def tearDown(self):
        self.connection.drop_database('generationkwh_test')

    def test_get_whenEmpty(self):
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = self.uri,
            curveProvider = self.curveProvider)

        self.assertEqual(
            list(m.get_kwh(
                date(2015,9,4),
                date(2015,9,5))),
            2*25*[0]
            )

    def test_get_afterUpdate(self):
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = self.uri,
            curveProvider = self.curveProvider)
        m.update_kwh(date(2015,9,4), date(2015,9,5))

        self.assertEqual(
            list(m.get_kwh(
                date(2015,9,4),
                date(2015,9,5))),
            [
                0,0,0,0,0,0,0,0,3,6,5,4,8,17,34,12,12,5,3,1,0,0,0,0,0,
                0,0,0,0,0,0,0,0,4,7,6,5,9,18,35,13,13,6,4,2,0,0,0,0,0,
            ])

    def test_lastDate_empty(self):
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = self.uri,
            curveProvider = self.curveProvider)
        self.assertEqual(m.lastMeasurementDate(), None)

    def test_lastDate_filled(self):
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = self.uri,
            curveProvider = self.curveProvider)
        m.update_kwh(date(2015,9,4), date(2015,9,5))

        self.assertEqual(m.lastMeasurementDate(), date(2015,9,5))

    def test_firstDate_empty(self):
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = self.uri,
            curveProvider = self.curveProvider)
        self.assertEqual(m.firstMeasurementDate(), None)

    def test_firstDate_filled(self):
        m = ProductionMeter(
            1,
            'meterName',
            'meterDescription',
            True,
            uri = self.uri,
            curveProvider = self.curveProvider)
        m.update_kwh(date(2015,9,4), date(2015,9,5))
        self.assertEqual(m.firstMeasurementDate(), date(2015,9,4))


unittest.TestCase.__str__ = unittest.TestCase.id

# vim: et ts=4 sw=4



from __future__ import absolute_import


class BaseProvider(object):
    """Base provider interface

    :param uri: URI to get the properties for the provider 
    """
    def __init__(self, uri):
        pass

    def __enter__(self):
        raise NotImplementedError()

    def __exit__(self, exc_type, exc_val, exc_tb):
        raise NotImplementedError()

    def insert(self, document):
        raise NotImplementedError()

    def get(self, collection, filters, fields=None):
        raise NotImplementedError()

    def disconnect(self):
        raise NotImplementedError()

class BaseProviderConnectionError(Exception):
	pass

class BaseProviderDownloadError(Exception):
	pass

class BaseProviderSyntaxError(Exception):
	pass

# vim: et ts=4 sw=4



from __future__ import absolute_import

from urlparse import urlparse as std_urlparse
from plantmeter.providers.base import BaseProvider, \
        BaseProviderConnectionError, BaseProviderDownloadError, BaseProviderSyntaxError

_AVAILABLE_PROVIDERS = {}


def register(name, cls):
    """Register a provider 

    :param name: Provider name
    :param class cls: Provider class
    """
    _AVAILABLE_PROVIDERS[name] = cls


def urlparse(url):
    url = std_urlparse(url)
    config = {
        'provider': url.scheme,
        'username': url.username,
        'password': url.password,
        'hostname': url.hostname,
        'port': url.port,
        'path': url.path.lstrip('/')
    }
    return config


def get_provider(uri):
    """Get the provider class by and URI.

    :param url: URI for identify a provider
    """
    config = urlparse(uri)
    provider = config['provider']
    if provider not in _AVAILABLE_PROVIDERS:
        raise Exception(
            'Provider {} is not available/registered'.format(provider)
        )
    return _AVAILABLE_PROVIDERS[provider]

# Import providers 
from plantmeter.providers.tfm import TFMProvider
from plantmeter.providers.csv import CSVProvider
from plantmeter.providers.monsol import MonsolProvider



import datetime
import pytz

tz = pytz.timezone('Europe/Madrid')

def asUtc(date):
    if date.tzinfo is None:
        return pytz.utc.localize(date)
    return date.astimezone(pytz.utc)

def toLocal(date):
    if date.tzinfo is None:
        return tz.localize(date)
    return date.astimezone(tz)

def parseLocalTime(string, isSummer=False, format="%Y-%m-%d %H:%M:%S"):
    naive = datetime.datetime.strptime(string, format)
    localized = tz.localize(naive)
    if not isSummer: return localized
    if localized.dst(): return localized
    onehour = datetime.timedelta(hours=1)
    lesser = tz.normalize(localized-onehour)
    return lesser if lesser.dst() else localized

def localisodate(string):
    return string and toLocal(datetime.datetime.strptime(string, "%Y-%m-%d"))

def naiveisodate(string):
    return string and datetime.datetime.strptime(string, "%Y-%m-%d")

def isodate(string):
    return string and datetime.datetime.strptime(string, "%Y-%m-%d").date()

def naiveisodatetime(string):
    return string and datetime.datetime.strptime(string, "%Y-%m-%d %H:%M:%S")

def localisodatetime(string):
    return string and toLocal(datetime.datetime.strptime(string, "%Y-%m-%d %H:%M:%S"))

def dateToLocal(date):
    # TODO: optimize dateToLocal
    return localisodate(str(date))

def assertDate(name, date):
    assert type(date)==datetime.date, (
        "{} should be a datetime.date but it is {}"
        .format(name, date))
    return date

def assertDateOrNone(name, date):
    if date is None: return date
    assert type(date)==datetime.date, (
        "{} should be a datetime.date or None but it is {}"
        .format(name, date))
    return date

def assertNaiveTime(name, date):
    assert type(date)==datetime.datetime, (
        "{} should be a datetime.datetime with no timezone (naive) but it is {}"
        .format(name, date))
    return date

def assertLocalDateTime(name, value):
    assert isinstance(value, datetime.datetime), (
        "{} should be a datetime".format(name))
    assert value.tzinfo, (
        "{} should have timezone".format(name))
    assert value.tzname() in ('CET','CEST'), (
        "{} has {} timezone".format(name, value.tzname()))

def addDays(date, ndays):
    resultday = date.date() + datetime.timedelta(days=ndays)
    naiveday = datetime.datetime.combine(resultday, datetime.time(0,0,0))
    return toLocal(naiveday)





import datetime
import ftplib
import pytz

from plantmeter.providers import BaseProvider, BaseProviderConnectionError, \
        BaseProviderDownloadError, BaseProviderSyntaxError, register, urlparse
from plantmeter.utils import daterange
from ..mongotimecurve import toLocal, parseLocalTime as mtcParseLocalTime

def parseLocalTime(string, isSummer):
    return mtcParseLocalTime(string, isSummer, format="%Y%m%d%H%M%S")


class MonsolProvider(BaseProvider):
    """Monsol Provider
    """

    def __init__(self, uri=None):
        if uri is None:
            uri = "monsol://path"
        super(MonsolProvider, self).__init__(uri)
        self.uri = uri
        self.res = urlparse(self.uri)

    def extract(self, content, start, end=None):
        def valid(line):
            items = line.split(';')

            n_items = 4
            return (items[0].isdigit() and
                items[1].isdigit() and
                items[2].isdigit() and
                items[3] in ['S', 'W']) if (len(items)==n_items and '' not in items) else False

        def parse(line):
            if not valid(line):
                raise BaseProviderSyntaxError('Data not valid: %s' % line)

            items = line.split(';')
            return {
                    'datetime': parseLocalTime(items[0], items[3]=='S'),
                    'ae': int(items[1])
                    }

        if not end:
            end = start

        header_offset = 1
        return [measure
            for date in daterange(start, end + datetime.timedelta(days=1))
                for measure in (parse(line) for idx,line in enumerate(content) if idx>header_offset)
                    if measure['datetime'].date() == date
            ]

    def download(self, date):
        # TODO: Dynamic plant identifier
        filename = '1_{date.year}{date.month:02d}{date.day:02d}.csv'.format(**locals())
        client = None
        try:
            client = ftplib.FTP(self.res['hostname'])
            client.login(self.res['username'], self.res['password'])
        except Exception as e:
            raise BaseProviderConnectionError(e)

        import StringIO
        from contextlib import closing
        with closing(StringIO.StringIO()) as sio:
            try:
                client.cwd(self.res['path'])
                client.retrbinary('RETR ' + filename, callback=sio.write)
            except Exception as e:
                raise BaseProviderDownloadError('Failed downloading %s' % filename)
            finally:
                client.quit()

            sio.seek(0)
            return sio.read().splitlines()

    def get(self, start, end):
        assert start is None or type(start)==datetime.date, (
            "start date should be a date but is {}".format(start))
        assert end is None or type(end)==datetime.date, (
            "start date should be a date but is {}".format(end))

        if not end:
            end = toLocal(datetime.datetime.now()).date()
        return [self.extract(self.download(date), date) for date in daterange(start, end)]

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

register("monsol", MonsolProvider)


# vim: et ts=4 sw=4



from datetime import timedelta, date

def daterange(start_date, end_date, **kwds):
    step = kwds or dict(days=1)
    if not end_date:
        end_date = start_date + timedelta(days=1)
    for n in range(int ((end_date - start_date).days)):
        yield start_date + n*timedelta(**step)



import os
import numpy as np

from plantmeter.providers import get_provider
from plantmeter.isodates import localisodate, assertDateOrNone, assertDate, dateToLocal
import datetime

"""
TODOs
+ Ordre
- Duplicats
- Gaps
+ Padding
"""

class Resource(object):
    id = None 
    name = None
    description = None
    enabled = None

    def __init__(self, id, name, description, enabled):
        self.id = id 
        self.name = name
        self.description = description
        self.enabled = enabled


class ProductionAggregator(Resource):
    def __init__(self, *args, **kwargs):
        self.plants = kwargs.pop('plants') if 'plants' in kwargs else []
        super(ProductionAggregator, self).__init__(*args, **kwargs)

    def get_kwh(self, start, end):

        assertDate('start', start)
        assertDate('end', end)

        return np.sum([
            plant.get_kwh(start, end)
            for plant in self.plants
            if plant.enabled
            ], axis=0)
    
    def update_kwh(self, start=None, end=None, notifier=None):

        assertDateOrNone('start', start)
        assertDateOrNone('end', end)

        return [(plant.id, plant.update_kwh(start, end, notifier)) for plant in self.plants]

    def firstMeasurementDate(self):
        return min([
            plant.firstMeasurementDate()
            for plant in self.plants
            if plant.enabled
            ])

    def lastMeasurementDate(self):
        return max([
            plant.lastMeasurementDate()
            for plant in self.plants
            if plant.enabled
            ])

class ProductionPlant(Resource):
    def __init__(self, *args, **kwargs):
        self.meters = kwargs.pop('meters') if 'meters' in kwargs else []
        super(ProductionPlant, self).__init__(*args, **kwargs)

    def get_kwh(self, start, end):
        assertDate('start', start)
        assertDate('end', end)

        return np.sum([
            meter.get_kwh(start, end)
            for meter in self.meters
            if meter.enabled
            ], axis=0)

    def update_kwh(self, start=None, end=None, notifier=None):

        assertDateOrNone('start', start)
        assertDateOrNone('end', end)

        return [(meter.id, meter.update_kwh(start, end, notifier)) for meter in self.meters]

    def lastMeasurementDate(self):
        return max([
            meter.lastMeasurementDate()
            for meter in self.meters
            if meter.enabled
            ])

    def firstMeasurementDate(self):
        return min([
            meter.firstMeasurementDate()
            for meter in self.meters
            if meter.enabled
            ])

class ProductionMeter(Resource):
    def __init__(self, *args, **kwargs):
        self.uri = kwargs.pop('uri') if 'uri' in kwargs else None
        self.lastcommit = kwargs.pop('lastcommit') if 'lastcommit' in kwargs else None
        if self.lastcommit:
            self.lastcommit = localisodate(self.lastcommit).date() + datetime.timedelta(days=1)
        self.curveProvider = kwargs.pop('curveProvider') if 'curveProvider' in kwargs else None
        super(ProductionMeter, self).__init__(*args, **kwargs)

    def get_kwh(self, start, end):

        assertDate('start', start)
        assertDate('end', end)

        return self.curveProvider.get(
            dateToLocal(start),
            dateToLocal(end),
            self.name,
            'ae'
            )

    def update_kwh(self, start=None, end=None , notifier=None):
        import datetime

        assertDateOrNone('start', start)
        assertDateOrNone('end', end)

        if start is None:
            start = self.lastcommit

        provider = get_provider(self.uri)
        updated = None
        with provider(self.uri) as remote:
            status = 'done'
            message = None
            try:
                for day in remote.get(start, end):
                    for measurement in day:
                        self.curveProvider.fillPoint(
                                name = self.name,
                                datetime = measurement['datetime'],
                                ae = measurement['ae']
                                )
                        updated = measurement['datetime']
            except Exception as e:
                status = 'failed'
                message = str(e)
            finally:
                if notifier:
                    notifier.push(self.id, status, message)
        return updated

    def lastMeasurementDate(self):
        result = self.curveProvider.lastFullDate(self.name)
        return result and result.date()
    
    def firstMeasurementDate(self):
        result = self.curveProvider.firstFullDate(self.name)
        return result and result.date()

# vim: et ts=4 sw=4



from __future__ import absolute_import
import os

#from osconf import config_from_environment

try:
    VERSION = __import__('pkg_resources') \
        .get_distribution(__name__).version
except Exception as e:
    VERSION = 'unknown'


#from plantmeter.logging import setup_logging
#logging_config = config_from_environment('PLANETMETER_LOGGING')
#logger = setup_logging(**logging_config)


_ROOT = os.path.abspath(os.path.dirname(__file__))


def get_data(path):
    return os.path.join(_ROOT, 'data', path)

from . import mongotimecurve
from . import resource 



# -*- coding: utf-8 -*-

import os
from osv import osv, fields
from mongodb_backend import osv_mongodb
from mongodb_backend.mongodb2 import mdbpool
from .erpwrapper import ErpWrapper

from datetime import datetime
from plantmeter.resource import ProductionAggregator, ProductionPlant, ProductionMeter 
from plantmeter.mongotimecurve import MongoTimeCurve, toLocal, asUtc
from plantmeter.isodates import isodate, naiveisodatetime

class GenerationkwhProductionAggregator(osv.osv):
    '''Implements generationkwh production aggregation '''

    _name = 'generationkwh.production.aggregator'


    _columns = {
        'name': fields.char('Name', size=50),
        'description': fields.char('Description', size=150),
        'enabled': fields.boolean('Enabled'),
        'plants': fields.one2many('generationkwh.production.plant', 'aggr_id', 'Plants')
    }

    _defaults = {
        'enabled': lambda *a: False
    }

    def get_kwh(self, cursor, uid, pid, start, end, context=None):
        '''Get production aggregation'''
   
        if not context:
            context = {}
        if isinstance(pid, list) or isinstance(pid, tuple):
            pid = pid[0]

        aggr = self.browse(cursor, uid, pid, context)
        _aggr = self._createAggregator(aggr, ['id', 'name', 'description', 'enabled'])
        return _aggr.get_kwh(start, end).tolist()

    def update_kwh(self, cursor, uid, pid, start=None, end=None, context=None):
        '''Update kWh measurements'''

        notifier = ProductionNotifierProvider(self, cursor, uid, context)
        if start > end: return
        if not context:
            context = {}
        if isinstance(pid, list) or isinstance(pid, tuple):
            pid = pid[0]

        args = ['id', 'name', 'description', 'enabled']
        aggr = self.browse(cursor, uid, pid, context)
        _aggr = self._createAggregator(aggr, args)
        self.updateLastCommit(cursor, uid, pid,
                _aggr.update_kwh(start, end, notifier))

    def updateLastCommit(self, cursor, uid, pid, lastcommits, context=None):
        '''Update last commit date'''

        if not context:
            context = {}
        if isinstance(pid, list) or isinstance(pid, tuple):
            pid = pid[0]

        plant = self.pool.get('generationkwh.production.plant')
        for plant_id, commits in lastcommits:
            plant.updateLastCommit(cursor, uid, plant_id, commits)

    def firstMeasurementDate(self, cursor, uid, pid, context=None):
        '''Get first measurement date'''

        if not context:
            context = {}
        if isinstance(pid, list) or isinstance(pid, tuple):
            pid = pid[0]

        args = ['id', 'name', 'description', 'enabled']
        aggr = self.browse(cursor, uid, pid, context)
        _aggr = self._createAggregator(aggr, args)
        date = _aggr.firstMeasurementDate()
        return date if date else None

    def lastMeasurementDate(self, cursor, uid, pid, context=None):
        '''Get last measurement date'''

        if not context:
            context = {}
        if isinstance(pid, list) or isinstance(pid, tuple):
            pid = pid[0]

        args = ['id', 'name', 'description', 'enabled']
        aggr = self.browse(cursor, uid, pid, context)
        _aggr = self._createAggregator(aggr, args)
        date = _aggr.lastMeasurementDate()
        return date if date else None

    def getNshares(self, cursor, uid, pid, context=None):
        '''Get number of shares'''

        if not context:
            context = {}
        if isinstance(pid, list) or isinstance(pid, tuple):
            pid = pid[0]

        aggr = self.browse(cursor, uid, pid, context)
        return sum([plant.nshares for plant in aggr.plants])

    def _createAggregator(self, aggr, args):
        def obj_to_dict(obj, attrs):
            return {attr: getattr(obj, attr) for attr in attrs}

        curveProvider = MongoTimeCurve(mdbpool.get_db(),
                'generationkwh.production.measurement')

        # TODO: Clean initialization method
        return ProductionAggregator(**dict(obj_to_dict(aggr, args).items() + 
            dict(plants=[ProductionPlant(**dict(obj_to_dict(plant, args).items() +
                dict(meters=[ProductionMeter(
                    **dict(obj_to_dict(meter, args + ['uri','lastcommit']).items() +
                    dict(curveProvider=curveProvider).items())) 
                for meter in plant.meters if meter.enabled]).items()))
            for plant in aggr.plants if plant.enabled]).items()))

GenerationkwhProductionAggregator()


class GenerationkwhProductionAggregatorTesthelper(osv.osv):
    '''Implements generationkwh production aggregation testhelper '''

    _name = 'generationkwh.production.aggregator.testhelper'
    _auto = False


    def get_kwh(self, cursor, uid, pid, start, end, context=None):
        production = self.pool.get('generationkwh.production.aggregator')
        return production.get_kwh(cursor, uid, pid,
                isodate(start), isodate(end), context)

    def update_kwh(self, cursor, uid, pid, start, end, context=None):
        production = self.pool.get('generationkwh.production.aggregator')
        return production.update_kwh(cursor, uid, pid,
                isodate(start), isodate(end), context)

    def firstMeasurementDate(self, cursor, uid, pid, context=None):
        production = self.pool.get('generationkwh.production.aggregator')
        result = production.firstMeasurementDate(cursor, uid, pid, context)
        return result and str(result)

    def lastMeasurementDate(self, cursor, uid, pid, context=None):
        production = self.pool.get('generationkwh.production.aggregator')
        result = production.lastMeasurementDate(cursor, uid, pid, context)
        return result and str(result)

    def getNshares(self, cursor, uid, pid, context=None):
        production = self.pool.get('generationkwh.production.aggregator')
        return production.getNshares(cursor, uid, pid, context)

    def clear_mongo_collections(self, cursor, uid, collections, context=None):
        for collection in collections:
            mdbpool.get_db().drop_collection(collection)

    def fillMeasurementPoint(self, cursor, uid, pointTime, name, value, context=None):
        curveProvider = MongoTimeCurve(mdbpool.get_db(),
                'generationkwh.production.measurement')
        curveProvider.fillPoint(
            datetime=toLocal(asUtc(naiveisodatetime(pointTime))),
            name=name,
            ae=value)


GenerationkwhProductionAggregatorTesthelper()


class GenerationkwhProductionPlant(osv.osv):

    _name = 'generationkwh.production.plant'
    _columns = {
        'name': fields.char('Name', size=50),
        'description': fields.char('Description', size=150),
        'enabled': fields.boolean('Enabled'),
        'nshares': fields.integer('Number of shares'),
        'aggr_id': fields.many2one('generationkwh.production.aggregator', 'Production aggregator',
                                  required=True),
        'meters': fields.one2many('generationkwh.production.meter', 'plant_id', 'Meters')
    }
    _defaults = {
        'enabled': lambda *a: False,
    }

    def updateLastCommit(self, cursor, uid, pid, lastcommits, context=None):
        '''Update last commit date'''

        if not context:
            context = {}
        if isinstance(pid, list) or isinstance(pid, tuple):
            pid = pid[0]

        meter = self.pool.get('generationkwh.production.meter')
        for meter_id, commits in lastcommits:
            meter.updateLastCommit(cursor, uid, meter_id, commits)

GenerationkwhProductionPlant()


class GenerationkwhProductionMeter(osv.osv):

    _name = 'generationkwh.production.meter'
    _columns = {
        'name': fields.char('Name', size=50),
        'description': fields.char('Description', size=150),
        'enabled': fields.boolean('Enabled'),
        'plant_id': fields.many2one('generationkwh.production.plant'),
        'uri': fields.char('Host', size=150, required=True),
        'lastcommit': fields.date('Last pull date')
        }
    _defaults = {
        'enabled': lambda *a: False,
    }

    def updateLastCommit(self, cursor, uid, pid, lastcommit, context=None):
        '''Update last commit date'''

        if not context:
            context = {}
        if isinstance(pid, list) or isinstance(pid, tuple):
            pid = pid[0]

        meter = self.pool.get('generationkwh.production.meter')
        if lastcommit:
            meter.write(cursor, uid, pid, {'lastcommit': lastcommit})

GenerationkwhProductionMeter()


class ProductionNotifierProvider(ErpWrapper):
    def __init__(self, erp, cursor, uid, pid, context=None):
        self.pid = pid
        super(ProductionNotifierProvider, self).__init__(erp, cursor, uid, context)

    def push(self, meter_id, status, message):
        notifier=self.erp.pool.get('generationkwh.production.notifier')
        return notifier.create(self.cursor, self.uid, {
            'meter_id': meter_id,
            'date_pull': datetime.now(),
            'status': status,
            'message': message
            })

class GenerationkwhProductionNotifier(osv.osv):
    _name = 'generationkwh.production.notifier'
    _rec_name = 'contract_id'

    _columns = {
        'meter_id': fields.many2one(
            'generationkwh.production.meter',
            'meter'
        ),
        'date_pull': fields.datetime('Last pull datetime'),
        'status': fields.selection([
            ('failed', 'Failed'),
            ('done', 'Done')]),
        'message': fields.char('Message', size=200)
    }

    _order = 'date_pull desc'
GenerationkwhProductionNotifier()


class GenerationkwhProductionNotifierTesthelper(osv.osv):
    _name = 'generationkwh.production.notifier.testhelper'
    _auto = False

    def push(self, cursor, uid, meter_id, status, message, context=None):
        from datetime import datetime

        notifier = ProductionNotifierProvider(self, cursor, uid, context)
        return notifier.push(meter_id, status, message)

GenerationkwhProductionNotifierTesthelper()


class GenerationkwhProductionMeasurement(osv_mongodb.osv_mongodb):

    _name = 'generationkwh.production.measurement'
    _order = 'timestamp desc'

    _columns = {
        'name': fields.integer('Plant identifier'), # NOTE: workaround due mongodb backend
        'create_at': fields.datetime('Create datetime'),
        'datetime': fields.datetime('Exported datetime'),
        'daylight': fields.char('Exported datetime daylight',size=1),
        'ae': fields.float('Exported energy (kWh)')
    }

    def search(self, cursor, uid, args, offset=0, limit=0, order=None,
               context=None, count=False):
        '''Exact match for name.
        In mongodb, even when an index exists, not exact
        searches for fields scan all documents in collection
        making it extremely slow. Making name exact search
        reduces dramatically the amount of documents scanned'''

        new_args = []
        for arg in args:
            if not isinstance(arg, (list, tuple)):
                new_args.append(arg)
                continue
            field, operator, value = arg
            if field == 'name' and operator != '=':
                operator = '='
            new_args.append((field, operator, value))
        return super(GenerationkwhProductionMeasurement,
                     self).search(cursor, uid, new_args,
                                  offset=offset, limit=limit,
                                  order=order, context=context,
                                  count=count)

GenerationkwhProductionMeasurement()

# vim: ts=4 sw=4 et



# -*- coding: utf8 -*-

class ErpWrapper(object):
    """
        Base class for classes that need to access OpenERP functionality
        but has to offer methods without OpenERP bullshit params.

        Such params are passed to the constructor instead so that
        the rest of the methods can access them internally as attributes.

        Example:

            class MyClass(ErpWrapper):
                def clientsByLang(self, lang):
                    Partner=self.erp.pool.get('res.partner')
                    return Partner.search(self.cursor, self.uid, [
                        ('lang','=',lang),
                        ], context=self.context)
    """

    def __init__(self, erp, cursor, uid, context=None):
        self.erp = erp
        self.cursor = cursor
        self.uid = uid
        self.context = context





from plantmeter.providers import BaseProvider, register, urlparse
from plantmeter.utils import daterange
import datetime
import xlrd

class TFMProvider(BaseProvider):
    """TFM Provider 
    """
    date_fmt = '&dia={date.day:02d}&mes={date.month:02d}&any={date.year:04d}'
    sheet_names = [u'Dades Instantaneas', u'Corba de carrega', u'Sondes', u'Dades Resumides']
    var_names = [u'Temps', u'Generaci\xf3', u'Temp. Modul', u'Temp. Ambient']
 
    def __init__(self, uri=None):
        if uri is None:
            uri = 'tfm://user:password@localhost:port/path'
        super(TFMProvider, self).__init__(uri)
        self.uri = uri
        self.res = urlparse(self.uri)

    def download(self, date):
        import requests
        return requests.get('http://' + self.res['hostname'] + '/' + self.res['path'] + 
            self.date_fmt.format(**locals())).content

    def extract(self, date, content):
        import xlrd
        sheets = [u'Dades Instantaneas', u'Corba de carrega', u'Sondes', u'Dades Resumides']
        book = xlrd.open_workbook(file_contents=content)
        if book.nsheets != 4 or set(book.sheet_names()) != set(self.sheet_names): 
            raise Exception('Failed parsing data')
       
        sheet = book.sheet_by_name('Dades Resumides')
        var_names = [cell.value for cell in sheet.row(0)]
        if sheet.ncols != 4 or sheet.nrows != 25 or set(var_names) != set(self.var_names): 
            raise Exception('Corruputed data')
        return [{'datetime': datetime.datetime.combine(date,datetime.time.min)+datetime.timedelta(hours=3),
                 'ae': sheet.row(idx)[1].value} 
                     for idx in range(1,25)] 

    def get(self, start, end=None):
        return [self.extract(date, self.download(date)) for date in daterange(start, end)]

    def disconnect(self):
        pass

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

register("tfm", TFMProvider)



#!/usr/bin/env python

import pymongo
from yamlns import namespace as ns
import numpy
import datetime

"""
+ More than one meassure
+ Different name ignored
+ Priority for newer meassuers the same hour
+ Summer daylight
+ Check mandatory fields
+ last meassure
+ first meassure
+ remove urlparse dependency on backends
+ Properly detect summer daylight change
- lastFullDate: proper implementation

"""

from .isodates import (
    asUtc,
    toLocal,
    parseLocalTime,
    assertLocalDateTime,
    tz,
    addDays,
    )


hoursPerDay=25

"""
- Spain daylight:
    - UTC offset
        - +1h in winter
        - +2h in summer
    - Hourly curves have
        - an padding to the left in winter: _ X X X ... X X
        - an padding to the right in summer: X X X ... X _
    - Summer to winter is last sunday of october
        - that day has no padding in neither side
        - It has two 2:00h, a summer and a winter one
    - Winter to summer is last sunday of march
        - that day has paddings on both sides
        - It has no 2:00h, next to winter 1:00h is summer 3:00h
"""


def dateToCurveIndex(start, localTime):
    """
        Maps a timezoned datetime to a hourly curve index starting
        at 'start' date. Time part of 'start' is ignored.
        Both times' timezone should match curve's timezone.

        A day in houry curve has 25 positions. Padding is added
        to keep same solar time in the same position across summer
        daylight saving shift.
    """
    ndays = (localTime.date()-start.date()).days
    winterOffset = 0 if localTime.dst() else 1
    return localTime.hour + hoursPerDay*ndays + winterOffset


def curveIndexToDate(start, index):
    """
        Maps an index withing an hourly curve starting at 'start' date
        into a local date.

        A day in houry curve has 25 positions. Padding is added
        to keep same solar time in the same position across summer
        daylight saving shift.
    """
    days = index//hoursPerDay
    localday = addDays(start, days)

    nextDay = tz.normalize(localday + datetime.timedelta(days=1))
    toWinterDls = localday.dst() and not nextDay.dst()
    toSummerDls = not localday.dst() and nextDay.dst()

    if not localday.dst(): index-=1
    hours = index%hoursPerDay
    if hours == 24 and not toWinterDls: return None
    if hours == 23 and toSummerDls: return None
    return tz.normalize(localday+datetime.timedelta(hours=hours))
   


class MongoTimeCurve(object):
    """Consolidates curve data in a mongo database"""

    def __init__(self, mongodb, collection):
        self.db = mongodb
        self.collectionName = collection
        self.collection = self.db[collection]

    def get(self, start, stop, filter, field, filling=None):

        assert start.tzinfo is not None, (
            "MongoTimeCurve.get called with naive (no timezone) start date")

        assert stop.tzinfo is not None, (
            "MongoTimeCurve.get called with naive (no timezone) stop date")

        ndays = (stop.date()-start.date()).days+1
        data = numpy.zeros(ndays*hoursPerDay, numpy.int)
        if filling :
            filldata = numpy.zeros(ndays*hoursPerDay, numpy.bool)
        filters = dict(
            name = filter,
            datetime = {
                '$gte': start,
                '$lt': addDays(stop,1)
            }
        )

        for x in (self.collection
                .find(filters, [field,'datetime'])
                .sort('create_at',pymongo.ASCENDING)
                ):
            point = ns(x)
            localTime = toLocal(asUtc(point.datetime))
            timeindex = dateToCurveIndex (start, localTime)
            data[timeindex]=point.get(field)
            if filling: filldata[timeindex]=True

        if filling: return data, filldata
        return data

    def fillPoint(self, **data):
        for requiredField in ('name', 'datetime'):
            if requiredField not in data:
                raise Exception("Missing '{}'".format(requiredField))

        assert data['datetime'].tzinfo is not None, (
            "MongoTimeCurve.fillPoint with naive (no timezone) datetime")

        counter = self.db['counters'].find_and_modify(
            {'_id': self.collectionName},
            {'$inc': {'counter': 1}}
        )
        data.update(
            create_at = datetime.datetime.now()
            )
        return self.collection.insert(data)

    def _firstLastDate(self, name, first=False):
        """returns the date of the first or last item of a given name"""
        order = pymongo.ASCENDING if first else pymongo.DESCENDING
        for point in (self.collection
                .find(dict(name=name))
                .sort('datetime', order)
                .limit(1)
                ):
            return toLocal(asUtc(ns(point).datetime)).replace(
                    hour=0,minute=0,second=0)
        return None

    def firstDate(self, name):
        """returns the date of the first item of a given name"""
        return self._firstLastDate(name, first=True)

    def lastDate(self, name):
        """returns the date of the last item of a given name"""
        return self._firstLastDate(name)

    def lastFullDate(self,name):
        # TODO: dumb implementation, if there is a single point consider it filled
        return self.lastDate(name)
    def firstFullDate(self,name):
        # TODO: dumb implementation, if there is a single point consider it filled
        return self.firstDate(name)
    
    def update(self, start, filter, field, data):
        """Updates the curve with new data"""

        assert start.tzinfo is not None, (
            "MongoTimeCurve.update called with naive (no timezone) start date")

        stop = start + datetime.timedelta(days=len(data)//hoursPerDay+1)
        oldData, filling = self.get(start, stop, filter, field, filling=True)
        for i,(bin,old,f) in enumerate(zip(data,oldData,filling)):
            curveDate = curveIndexToDate(start, i)
            if curveDate is None: continue
            if bin == old: continue
            self.fillPoint(
                datetime=curveDate,
                name=filter,
                **dict([(field, bin)])
                )



# vim: et ts=4 sw=4



#!/usr/bin/env python
from setuptools import setup, find_packages

readme = open("README.rst").read()

setup(
	name = "plantmeter",
	version = "1.2",
	description =
		"OpenERP module and library to manage multisite energy generation",
	author = "Som Energia SCCL",
	author_email = "info@somenergia.coop",
	url = 'https://github.com/Som-Energia/plantmeter',
	long_description = readme,
	license = 'GNU General Public License v3 or later (GPLv3+)',
	packages=find_packages(exclude=['*[tT]est*']),
	scripts=[
		],
	install_requires=[
        'pymongo<3.0',
        'numpy',
        'xlrd',
        'yamlns',
        'pytz',
        'erppeek',
        'consolemsg',
        'mock',
        'b2btest',
	],
	include_package_data = True,
	test_suite = 'plantmeter',
#	test_runner = 'colour_runner.runner.ColourTextTestRunner',
	classifiers = [
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Intended Audience :: Developers',
		'Development Status :: 2 - Pre-Alpha',
		'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
		'Operating System :: OS Independent',
	],
)

